window.Gleam.initSearch([{"doc":"messua","title":"messua","content":"# messua\n\nA collection of convenience types and functions that might amount to a\nsmall web framework sitting atop [mist](https://hexdocs.pm/mist/).\n\n[![Package Version](https://img.shields.io/hexpm/v/messua)](https://hex.pm/packages/messua)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/messua/)\n\n_Messua_ is a [genus of spider](https://en.wikipedia.org/wiki/Messua_(spider)),\nwhich apparently was itself named after\n[a character from _The Jungle Book_](https://en.wikipedia.org/wiki/List_of_The_Jungle_Book_characters#Messua).\nPlease do not take this as any sort of endorsement of any orientalist or\ncolonialist themes that may appear in Kipling&#39;s work; I just wanted a\nname vaguely related to webs that probably wouldn&#39;t already be taken.\n\n## Introduction\n\nWhen a piece of technology focuses on a particular solution or style in\nthe face of many alternatives, it is in fashion these days to refer to that\npiece of technology as being &quot;opinionated&quot;, particularly when describing\na programming language, framework, or library. I wouldn&#39;t say that Messua\nis _opinionated_ so much as I&#39;d say it _sees the point_ of doing things a\ncertain way, and is willing to give that a go. It is based largely on a\ncouple of convenient patterns I have found myself using when implementing\nbackends in Rust and Gleam:\n\n1. A common `Result(Response, Error)` type returned by handlers, for which\n   the `Error` variant gets automatically converted to a response.\n2. A common `Request` type (injected[^injected] with some server state),\n   passed &quot;down&quot; through layers of middleware, with the aforementioned\n   `Result` type bubbling back up.\n3. Functions that couple this `Result` type with Gleam&#39;s `use` sugar to\n   mimic early returns of `Error` variants that then get handled with a\n   minimum of ceremony, permitting focus on business logic and an\n   uncluttered happy path.\n\n[^injected]: I use the term &quot;injected&quot;, but really it&#39;s more like,\n             &quot;bundled with&quot;. Under the hood, the `MRequest` is just a\n             wrapper that holds the underlying request (the `gleam_http`\n             type) and your state.\n\nThis library owes a lot to a couple of Rust crates: the\n[`axum`](https://docs.rs/axum/latest/axum/) web server, and the\n[`tower`](https://docs.rs/tower/latest/tower/index.html) middleware crate,\nparticularly its\n[`Layer`](https://docs.rs/tower/latest/tower/trait.Layer.html) trait.\nYou might even call this package a sort of cozily-sized Gleam version of\nsome of the `axum`/`tower` ecosystem.\n\n## Approach\n\nMessua provides a sort of `mist` harness into which it hooks your handler\nfunction (and any `Layer`s you might have on top of it). It provides your\nstack with an `MRequest` and expects it to return an `MResponse` (which is\njust an alias for a `Result` whose `Ok` variant is a\n`gleam/http/response.Response`, and whose `Error` variant is `messua/err.Err`,\nwhich gets turned into an actual HTTP response by the harness). It then\nprovides you with a slew of convenience functions for extracting parts of\nrequests, routing, and generating responses.\n\nMessua also provides wrappers around some key things from `mist` and\n`gleam_http`; the idea here is that it&#39;s easier and more convenient to\nimport and use a single package, rather than having to mess around in the\nguts of three separate packages, wondering which one has the functionality\nyou want.\n\n## Examples\n\nThere is a growing list of examples in the\n[examples directory](https://codeberg.org/d2718/messua/src/branch/main/examples);\nhere&#39;s the simplest one:\n\n```gleam\nimport messua\nimport messua/ok\nimport messua/rr.{type MRequest, type MResponse}\n\nfn handler(_req: MRequest(Nil)) -&gt; MResponse {\n  ok.ok()\n  |&gt; ok.with_text_body(&quot;Hello, World!\\n&quot;)\n  |&gt; Ok()\n}\n\npub fn main() {\n  messua.default()\n  |&gt; messua.with_binding(&quot;localhost&quot;)\n  |&gt; messua.with_http(8080)\n  |&gt; messua.start(handler)\n}\n```\n\nA `gleam run` should be all you need to get the server started; then you can\nquery it and get a response:\n\n```text\n~/ $ curl localhost:8080\nHello, World!\n```\n","url":"index.html"},{"doc":"messua/err","title":"Err","content":"pub type Err {\n  Err(\n    status: Int,\n    headers: List(http.Header),\n    log_msg: Option(StringBuilder),\n    body_msg: Option(StringBuilder),\n  )\n}\n The `Error` variant of the MResponse type.\n\n It will be converted to an actual `gleam/http.Response` once it&#39;s out\n of your handler&#39;s hands. You can optionally give it &quot;log message&quot;\n (distinct from the body message and not returned with the\n response---intended only for a little visibility of server errors) that\n will be logged by the log `Layer` from the `messua/mware` module.\n (Alternatively, of course, you can write your own logging `Layer` that\n uses these.)\nErr(\n  status: Int,\n  headers: List(http.Header),\n  log_msg: Option(StringBuilder),\n  body_msg: Option(StringBuilder),\n)\n You shouldn&#39;t need to use this constructor yourself; there are\n plenty of functions for building these more conveniently.\nstatus\n&lt;p&gt;Response status code. This will probably be in the 4xx or 5xx range.\nMany of the &lt;code&gt;messua/handle.require_xxx&lt;/code&gt; functions return 400 errors\non failure.&lt;/p&gt;\n\nheaders\n&lt;p&gt;Any additional headers you want to provide; these are not generally\nrequired.&lt;/p&gt;\n\nlog_msg\n&lt;p&gt;An optional diagnostic message for the server administrator’s benefit.&lt;/p&gt;\n\nbody_msg\n&lt;p&gt;An optional message to send as the body of the response.&lt;/p&gt;\n\nSynonyms:\nmessua/err.Err\nmessua/err Err","url":"messua/err.html#Err"},{"doc":"messua/err","title":"log","content":"pub fn log(e: Err, msg: List(String)) -&gt; Err\n Add a log message to the given response.\n\n The message is supplied as a list of strings that will get joined\n with no intervening content.\nSynonyms:\nmessua/err.log\nmessua/err log","url":"messua/err.html#log"},{"doc":"messua/err","title":"new","content":"pub fn new(status: Int) -&gt; Err\n Instantiate a new message with the given status code.\n\n See also [`messua/errs`](errs.html) for some functions to instantiate\n these more mnemonically.\nSynonyms:\nmessua/err.new\nmessua/err new","url":"messua/err.html#new"},{"doc":"messua/err","title":"to_response","content":"pub fn to_response(\n  e: Err,\n  logger: fn(StringBuilder) -&gt; Nil,\n) -&gt; Response(ResponseData)\n This function is used by the handler harness to convert the `Err`\n to a proper `Response` before shooting it back down the pipe.\nSynonyms:\nmessua/err.to_response\nmessua/err to_response","url":"messua/err.html#to_response"},{"doc":"messua/err","title":"with_header","content":"pub fn with_header(e: Err, name: String, value: String) -&gt; Err\n Add the given `name: value` header to the error response.\nSynonyms:\nmessua/err.with_header\nmessua/err with_header","url":"messua/err.html#with_header"},{"doc":"messua/err","title":"with_message","content":"pub fn with_message(e: Err, msg: List(String)) -&gt; Err\n Add a body message to the given response.\n\n The message is supplied as a list of strings that will get joined\n with no intervening content, but will have a `\\n` suffixed.\nSynonyms:\nmessua/err.with_message\nmessua/err with_message","url":"messua/err.html#with_message"},{"doc":"messua/err","title":"messua/err","content":" An error type made to be converted into an error response.\n\n This allows for streamlined handling of error responses.\n\n ```gleam\n import messua/err\n import messua/errs\n import messua/handle\n import messua/ok\n import messua/rr.{type MRequest, type MResponse}\n \n import my_mod.{type ServerState}\n \n fn update_record(req: MRequest(ServerState)) -&gt; MResponse {\n   // If there&#39;s no Authorization: header, this returns an `Err` that\n   // becomes a 400 response.\n   use auth_key &lt;- handle.require_header(req, &quot;authorization&quot;)\n   \n   // If auth fails, we construct an error response ourselves.\n   use user_id &lt;- result.try(\n     case rr.state(req) |&gt; my_mod.verify_auth(auth_key) {\n       my_mod.AuthFailure -&gt; errs.unauthorized() |&gt; Error()\n       my_mod.AuthSuccess(user_id) -&gt; Ok(user_id)\n     }\n   )\n   \n   // If there&#39;s not a properly-formed request body, this returns an\n   // `Err` that becomes a 400.\n   use new_record &lt;- handle.require_json_body(req, my_mod.record_decoder)\n   \n   // If there&#39;s an error in our business logic, we return an `Err`\n   // that becomes a 500, and add the error message to be logged.\n   use response_data &lt;- result.try(\n     rr.state(req)\n     |&gt; my_mod.update_record(user_id, new_record)\n     |&gt; result.map_error(fn(msg) { errs.error() |&gt; err.log([msg]) })\n   )\n   \n   let body = my_mod.encode_response_data(response_data)\n   \n   ok.ok()\n   |&gt; ok.with_json_body(body)\n   |&gt; Ok()\n }\n ```","url":"messua/err.html"},{"doc":"messua/rr","title":"Handler","content":"pub type Handler(state) =\n  fn(MRequest(state)) -&gt; MResponse\n Naturally, a function that accepts an `MRequest` and returns an\n `MResponse` is considered a request `Handler`.\n\nSynonyms:\nmessua/rr.Handler\nmessua/rr Handler","url":"messua/rr.html#Handler"},{"doc":"messua/rr","title":"Layer","content":"pub type Layer(state) =\n  fn(MRequest(state), Handler(state)) -&gt; MResponse\n A `Layer` is a `Handler` with another &quot;inner&quot; `Handler` it can optionally\n pass an incoming request through. This inner handler may itself be\n another layer; it is possible to form stacks arbitrarily deep this way\n (see the [Stack\n example](https://codeberg.org/d2718/messua/src/branch/main/examples/stack)).\n\n (The inspiration for this particular type is the\n [`tower-layer`](https://docs.rs/tower-layer/0.3.3/tower_layer/) Rust\n crate.)\n\n ## Examples\n \n A layer that logs requests to stdout might look like this:\n\n \n import gleam/int\n import gleam/io\n import gleam/string_builder\n\n import gleam/http\n\n fn handler(req: MRequest(State)) -&gt; MResponse {\n // ...Routing and handling goes here.  \n }\n \n fn log_layer(req: MRequest(State), next: Handler(State)) -&gt; MResponse {\n   // Get source, method, path from request.\n   let source = case info(req) {\n     Ok(info) -&gt; string_builder.from_strings([\n       string.inspect(info.ip_address), &quot;:&quot;, int.to_string(info.port)\n     ])\n     Error(_) -&gt; string_builder.from_string(&quot;[ source ??? ]&quot;)\n   }\n   \n   let inner_req = inner(req)\n   let url = string_builder.from_strings([\n     string.inspect(inner_req.method),\n     &quot; &quot;,\n     string.inspect(inner_req.path)\n   ])\n\n   // Call inner handling function, get response.\n   let mresp = next(req)\n\n   // Note response status.\n   let result_msg = case mresp {\n     Ok(resp) -&gt; string_builder.from_string(int.to_string(resp.status))\n     Err(e) -&gt; string_builder.from_string(int.to_string(e.status))\n   }\n\n   // Print a request log entry with the source, method, path fo the\n   // request and the status code of the associated response.\n   string_builder.append(source, &quot; &quot;)\n   |&gt; string_builder.append_builder(url)\n   |&gt; string_builder.append(&quot; &quot;)\n   |&gt; string_builder.append(result_msg)\n   |&gt; string_builder.to_string()\n   |&gt; io.println()\n\n   // Return the response.\n   mresp\n }\n\n messua.default()\n |&gt; messua.start(log_layer(_, handler))\n \n\nSynonyms:\nmessua/rr.Layer\nmessua/rr Layer","url":"messua/rr.html#Layer"},{"doc":"messua/rr","title":"MRequest","content":"pub opaque type MRequest(state)\n A request with some user-defined server state.\n\n The type parameter is the type of the state object that gets\n injected into (or bundled with) each incoming request.\n\nSynonyms:\nmessua/rr.MRequest\nmessua/rr MRequest","url":"messua/rr.html#MRequest"},{"doc":"messua/rr","title":"MResponse","content":"pub type MResponse =\n  Result(response.Response(mist.ResponseData), err.Err)\n A result type with Errors that will automatically become responses.\n\nSynonyms:\nmessua/rr.MResponse\nmessua/rr MResponse","url":"messua/rr.html#MResponse"},{"doc":"messua/rr","title":"info","content":"pub fn info(req: MRequest(a)) -&gt; Result(ConnectionInfo, Nil)\n Get the underlying connection&#39;s\n [`mist.ConnectionInfo`](https://hexdocs.pm/mist/mist.html#ConnectionInfo).\n\n Consider [`handle.get_client()`](handle.html#get_client) or\n [`handle.require_client()`](handle.html#require_client) for something\n in this namespace  that returns equivalent information.\nSynonyms:\nmessua/rr.info\nmessua/rr info","url":"messua/rr.html#info"},{"doc":"messua/rr","title":"inner","content":"pub fn inner(req: MRequest(a)) -&gt; Request(Connection)\n Access the underlying\n [`gleam/http/request.Request`](https://hexdocs.pm/gleam_http/gleam/http/request.html#Request).\nSynonyms:\nmessua/rr.inner\nmessua/rr inner","url":"messua/rr.html#inner"},{"doc":"messua/rr","title":"make_mist_handler","content":"pub fn make_mist_handler(\n  state: a,\n  handle: fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; fn(Request(Connection)) -&gt; Response(ResponseData)\n [`messau.start()`](../messua.html#start) uses this internally. You don&#39;t\n need to use it, but Gleam&#39;s visibility rules aren&#39;t granular enough\n to hide it fron you.\nSynonyms:\nmessua/rr.make_mist_handler\nmessua/rr make_mist_handler","url":"messua/rr.html#make_mist_handler"},{"doc":"messua/rr","title":"stack","content":"pub fn stack(\n  base: fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n  layers: List(\n    fn(\n      MRequest(a),\n      fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n    ) -&gt; Result(Response(ResponseData), Err),\n  ),\n) -&gt; fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err)\n Compose a stack of nested `Layer`s into a `Handler` that can be\n passed to `messua.start()`.\n \n The front of the list is the &quot;bottom&quot; of the stack; the end of the\n list is the top.\n\n ## Examples\n\n See [the Stack\n example](https://codeberg.org/d2718/messua/src/branch/main/examples/mutstate)\n in the repository for an expository example.\nSynonyms:\nmessua/rr.stack\nmessua/rr stack","url":"messua/rr.html#stack"},{"doc":"messua/rr","title":"state","content":"pub fn state(req: MRequest(a)) -&gt; a\n Retrieve the state that&#39;s been injected into the `MRequest`.\nSynonyms:\nmessua/rr.state\nmessua/rr state","url":"messua/rr.html#state"},{"doc":"messua/rr","title":"messua/rr","content":" Requests and Responses.","url":"messua/rr.html"},{"doc":"messua","title":"Config","content":"pub opaque type Config(state)\n The type we use for building our server configration.\n\nSynonyms:\nmessua.Config\nmessua Config","url":"messua.html#Config"},{"doc":"messua","title":"HttpsConfig","content":"pub type HttpsConfig {\n  HttpsConfig(port: Int, cert_path: String, key_path: String)\n}\n HTTPS configuration struct to pass to `with_https()`.\nHttpsConfig(port: Int, cert_path: String, key_path: String)\n\nport\n&lt;p&gt;Port on which to listen for incoming secure requests.&lt;/p&gt;\n\ncert_path\n&lt;p&gt;Path to your SSL certificate file. This will probably be something\nlike &lt;code&gt;cert.pem&lt;/code&gt;.&lt;/p&gt;\n\nkey_path\n&lt;p&gt;Path to your SSL private key file. This will probably be something\nlike &lt;code&gt;privkey.pem&lt;/code&gt;.&lt;/p&gt;\n\nSynonyms:\nmessua.HttpsConfig\nmessua HttpsConfig","url":"messua.html#HttpsConfig"},{"doc":"messua","title":"default","content":"pub fn default() -&gt; Config(Nil)\n Return a new default server configuration:\n\n * Listens on `0.0.0.0`\n * Serves HTTP requests on port 80\nSynonyms:\nmessua.default\nmessua default","url":"messua.html#default"},{"doc":"messua","title":"default_https","content":"pub fn default_https(\n  cert_path: String,\n  key_path: String,\n) -&gt; HttpsConfig\n HTTPS configuration serving on the default port (443).\nSynonyms:\nmessua.default_https\nmessua default_https","url":"messua.html#default_https"},{"doc":"messua","title":"start","content":"pub fn start(\n  cfg: Config(a),\n  handler: fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Nil\n Start the server with your given handler function.\n\n This function doesn&#39;t return.\n\n ## Safety\n\n If you have misconfigured your server in a fatal way (such as specifying\n an invalid or noexistent SSL certificate file, or setting it to listen\n for neither HTTP _or_ HTTPS requests), Messua will just crash right away.\n This seems like a better option than limping along while you wonder why\n nothing is working correctly.\nSynonyms:\nmessua.start\nmessua start","url":"messua.html#start"},{"doc":"messua","title":"with_binding","content":"pub fn with_binding(cfg: Config(a), iface: String) -&gt; Config(a)\n Bind to the given interface.\n\n `iface` can be a valid IPv4 or IPv6 address, or `&quot;localhost&quot;`. An\n invalid value will crash the server when it starts.\nSynonyms:\nmessua.with_binding\nmessua with_binding","url":"messua.html#with_binding"},{"doc":"messua","title":"with_http","content":"pub fn with_http(cfg: Config(a), port: Int) -&gt; Config(a)\n Serve HTTP requests on a port other than 80.\nSynonyms:\nmessua.with_http\nmessua with_http","url":"messua.html#with_http"},{"doc":"messua","title":"with_https","content":"pub fn with_https(\n  cfg: Config(a),\n  https_config: HttpsConfig,\n) -&gt; Config(a)\n Serve secure HTTPS requests.\nSynonyms:\nmessua.with_https\nmessua with_https","url":"messua.html#with_https"},{"doc":"messua","title":"with_state","content":"pub fn with_state(cfg: Config(a), state: b) -&gt; Config(b)\n The value to be injected into each request as &quot;state&quot; so your handlers\n have access to it.\n\n This might be a database handle or some type of configuration struct.\nSynonyms:\nmessua.with_state\nmessua with_state","url":"messua.html#with_state"},{"doc":"messua","title":"without_http","content":"pub fn without_http(cfg: Config(a)) -&gt; Config(a)\n Do not serve HTTP requests.\n\n The default configuration serves HTTP on port 80, so if you _don&#39;t_\n want to serve insecure requests, you need to use this.\nSynonyms:\nmessua.without_http\nmessua without_http","url":"messua.html#without_http"},{"doc":"messua","title":"messua","content":" The root module is for server configuration.\n\n The `with_` functions (and the one `without_` function) are builder\n pattern functions.\n\n ```gleam\n import messua\n import my_module\n\n fn main() {\n   messua.default()\n   |&gt; messua.with_binding(&quot;localhost&quot;)\n   |&gt; messua.with_http(8080)\n   |&gt; messua.start(my_module.my_handler)\n }\n ```","url":"messua.html"},{"doc":"messua/ok","title":"compress_body","content":"pub fn compress_body(\n  resp: Response(ResponseData),\n) -&gt; Response(ResponseData)\n Compress a response body using `deflate`.\n\n This sets the Content-Encoding header, but you should still make sure\n a Content-Type has been set.\n\n Currently, this only works with bodies you have set yourself (i.e.,\n using one of the `with_xxx_body()` functions); it will not have any\n effect on files sent with\n [`serve_file()`](#serve_file) or\n [&#39;serve_dir()&#39;](#serve_dir).\nSynonyms:\nmessua/ok.compress_body\nmessua/ok compress_body","url":"messua/ok.html#compress_body"},{"doc":"messua/ok","title":"ok","content":"pub fn ok() -&gt; Response(ResponseData)\n Create a new, empty 200 OK response.\nSynonyms:\nmessua/ok.ok\nmessua/ok ok","url":"messua/ok.html#ok"},{"doc":"messua/ok","title":"serve_dir","content":"pub fn serve_dir(\n  dir: String,\n  subsegments: List(String),\n) -&gt; Result(Response(ResponseData), Err)\n Serve a file from the given directory.\n\n The `subsegments` argument should be a list of &quot;path segments&quot; that\n to be appended to the `dir` argument to form the final on-disk path\n of the file to send. If the path isn&#39;t found, or there&#39;s otherwise\n a problem sending the file, a 404 error will be sent.\n\n Like [`serve_file()`](#serve_file), this does not (as of yet) work\n in conjunction with [`compress_body()`](#compress_body).\n\n ## Examples\n\n This is meant to be used in conjunction with\n [`handle.path_segments()`](handle.html#path_segments) and pattern\n matching:\n\n \n import messua/errs\n import messua/handle\n import messua.ok\n\n fn handle_api_request(req: MRequest, segments: List(String)) -&gt; MResponse {\n   // Your important proprietary SaaS biznass goes here.\n }\n\n fn router(req: MRequest(state)) -&gt; MResponse {\n   case handle.path_segments(req) {\n      [&quot;api&quot;, ..rest] -&gt; handle_api_request(req, rest)\n      [&quot;static&quot;, ..rest] -&gt; ok.serve_dir(&quot;/app/assets/static&quot;, rest)\n      [] -&gt; ok.serve_file(&quot;/app/assets/index.html&quot;)\n      _ -&gt; errs.not_found()\n   }\n}\n \nSynonyms:\nmessua/ok.serve_dir\nmessua/ok serve_dir","url":"messua/ok.html#serve_dir"},{"doc":"messua/ok","title":"serve_file","content":"pub fn serve_file(\n  path: String,\n) -&gt; Result(Response(ResponseData), Err)\n Serve the given file as the body of the request.\n\n Content-Type will be guessed and set by file extension using the\n [`marceau`](https://hexdocs.pm/marceau/) package; if the supplied\n path isn&#39;t found (or can&#39;t be read), a 404 error will be sent instead.\n\n Please note that [`compress_body()`](#compress_body) will _not_ work\n with this function (it won&#39;t cause an _error_, it just won&#39;t compress\n anything). This function uses some lower-level Erlang magic\n to send the file with low overhead, and I don&#39;t yet know enough\n Erlang[^enough] to compress data at that level.\n\n [^enough]: And by &quot;enough&quot;, I mean &quot;any&quot;.\n\n TODO: Learn some Erlang and do better.\nSynonyms:\nmessua/ok.serve_file\nmessua/ok serve_file","url":"messua/ok.html#serve_file"},{"doc":"messua/ok","title":"with_binary_body","content":"pub fn with_binary_body(\n  resp: Response(ResponseData),\n  body: BytesBuilder,\n) -&gt; Response(ResponseData)\n Add a body of raw bytes.\n\n This sets the Content-Type to `application/octet-stream`; use\n `with_header()` if you want a different Content-Type.\nSynonyms:\nmessua/ok.with_binary_body\nmessua/ok with_binary_body","url":"messua/ok.html#with_binary_body"},{"doc":"messua/ok","title":"with_header","content":"pub fn with_header(\n  resp: Response(ResponseData),\n  name: String,\n  value: String,\n) -&gt; Response(ResponseData)\n Add the given header to the response.\nSynonyms:\nmessua/ok.with_header\nmessua/ok with_header","url":"messua/ok.html#with_header"},{"doc":"messua/ok","title":"with_json_body","content":"pub fn with_json_body(\n  resp: Response(ResponseData),\n  body: Json,\n) -&gt; Response(ResponseData)\n Add a [`Json`](https://hexdocs.pm/gleam_json/gleam/json.html#Json)\n body (as produced by the `gleam_json` package).\nSynonyms:\nmessua/ok.with_json_body\nmessua/ok with_json_body","url":"messua/ok.html#with_json_body"},{"doc":"messua/ok","title":"with_text_body","content":"pub fn with_text_body(\n  resp: Response(ResponseData),\n  body: String,\n) -&gt; Response(ResponseData)\n Add a plain-text body.\nSynonyms:\nmessua/ok.with_text_body\nmessua/ok with_text_body","url":"messua/ok.html#with_text_body"},{"doc":"messua/ok","title":"messua/ok","content":" Helpers for building responses.\n\n All body-adding functions set an appropriate `Content-Type`\n header unless otherwise noted.\n\n You shouldn&#39;t need to set the `Content-Length` header; that&#39;ll get\n set properly and automatically.","url":"messua/ok.html"},{"doc":"messua/both","title":"append_body_text","content":"pub fn append_body_text(\n  resp: Result(Response(ResponseData), Err),\n  body: String,\n) -&gt; Result(Response(ResponseData), Err)\n Append the given string to the response body.\n\n If the response doesn&#39;t already have a body, it will be given one; if\n the response has a [&quot;file&quot; body](ok.html#serve_file), this will\n replace it.\nSynonyms:\nmessua/both.append_body_text\nmessua/both append_body_text","url":"messua/both.html#append_body_text"},{"doc":"messua/both","title":"with_header","content":"pub fn with_header(\n  resp: Result(Response(ResponseData), Err),\n  name: String,\n  value: String,\n) -&gt; Result(Response(ResponseData), Err)\n Add the given `Name: Value` header to the response.\nSynonyms:\nmessua/both.with_header\nmessua/both with_header","url":"messua/both.html#with_header"},{"doc":"messua/both","title":"with_text_body","content":"pub fn with_text_body(\n  resp: Result(Response(ResponseData), Err),\n  body: String,\n) -&gt; Result(Response(ResponseData), Err)\n Set the response body to the given text string, along with the\n appropriate Content-Type.\n\n If the response already has a body, this will replace it.\nSynonyms:\nmessua/both.with_text_body\nmessua/both with_text_body","url":"messua/both.html#with_text_body"},{"doc":"messua/both","title":"messua/both","content":" Operations that can be applied symmetrically to either variant of\n an `MResponse`.","url":"messua/both.html"},{"doc":"messua/errs","title":"bad_request","content":"pub fn bad_request() -&gt; Err\n Error 400: Bad Request\n\n The request is missing some information, like a header value or a\n well-formed body that the server needs to respond to it.\nSynonyms:\nmessua/errs.bad_request\nmessua/errs bad_request","url":"messua/errs.html#bad_request"},{"doc":"messua/errs","title":"content_too_large","content":"pub fn content_too_large() -&gt; Err\n Error 413: Content Too Large\n\n For when the client has sent a larger request body than the server\n would like.\nSynonyms:\nmessua/errs.content_too_large\nmessua/errs content_too_large","url":"messua/errs.html#content_too_large"},{"doc":"messua/errs","title":"error","content":"pub fn error() -&gt; Err\n Error 500: Internal Server Error\n\n The generic catch-all error to send when the server has encountered\n a problem.\nSynonyms:\nmessua/errs.error\nmessua/errs error","url":"messua/errs.html#error"},{"doc":"messua/errs","title":"forbidden","content":"pub fn forbidden() -&gt; Err\n Error 403: Forbidden\n\n The server knows who the client is, it just won&#39;t let them do what\n they&#39;re asking.\nSynonyms:\nmessua/errs.forbidden\nmessua/errs forbidden","url":"messua/errs.html#forbidden"},{"doc":"messua/errs","title":"method_not_allowed","content":"pub fn method_not_allowed() -&gt; Err\n Error 405: Method Not Allowed\n\n It&#39;s considered good practice to inclued an `Allow` header in the\n response listing the methods you do allow.\nSynonyms:\nmessua/errs.method_not_allowed\nmessua/errs method_not_allowed","url":"messua/errs.html#method_not_allowed"},{"doc":"messua/errs","title":"not_found","content":"pub fn not_found() -&gt; Err\n Error 404: Not Found\n\n This one has become a meme that even nontechnical people use.\nSynonyms:\nmessua/errs.not_found\nmessua/errs not_found","url":"messua/errs.html#not_found"},{"doc":"messua/errs","title":"not_implemented","content":"pub fn not_implemented() -&gt; Err\n Error 501: Not Implemented\n\n This is a good thing to return when you haven&#39;t finished building\n your service.\nSynonyms:\nmessua/errs.not_implemented\nmessua/errs not_implemented","url":"messua/errs.html#not_implemented"},{"doc":"messua/errs","title":"teapot","content":"pub fn teapot() -&gt; Err\n Error 418: I&#39;m a Teapot\n\n It&#39;s probably acceptable to return this response for any request that\n just seems wildly out of order.\nSynonyms:\nmessua/errs.teapot\nmessua/errs teapot","url":"messua/errs.html#teapot"},{"doc":"messua/errs","title":"too_many_requests","content":"pub fn too_many_requests() -&gt; Err\n Error 429: Too Many Requests\n\n The server has decided to rate limit the client.\nSynonyms:\nmessua/errs.too_many_requests\nmessua/errs too_many_requests","url":"messua/errs.html#too_many_requests"},{"doc":"messua/errs","title":"unauthorized","content":"pub fn unauthorized() -&gt; Err\n Error 401: Unauthorized\n\n The client hasn&#39;t convinced the server they are who they say they are.\nSynonyms:\nmessua/errs.unauthorized\nmessua/errs unauthorized","url":"messua/errs.html#unauthorized"},{"doc":"messua/errs","title":"messua/errs","content":" A collection of convenience functions for creating `Err`s.\n\n Currently, this is just a bunch of functions with mnemonic names in\n case that&#39;s easier for you, or clearer to future you, than remembering\n specific HTTP status codes.\n\n ```gleam\n fn handle_api_request(_req: MRequest(State)) -&gt; MResponse {\n   errs.not_implemented() |&gt; Error()\n }\n ```","url":"messua/errs.html"},{"doc":"messua/handle","title":"IpAddr","content":"pub type IpAddr {\n  Ipv4(Int, Int, Int, Int)\n  Ipv6(Int, Int, Int, Int, Int, Int, Int, Int)\n}\n\nIpv4(Int, Int, Int, Int)\n\n\nIpv6(Int, Int, Int, Int, Int, Int, Int, Int)\n\n\nSynonyms:\nmessua/handle.IpAddr\nmessua/handle IpAddr","url":"messua/handle.html#IpAddr"},{"doc":"messua/handle","title":"Method","content":"pub type Method =\n  http.Method\n Reexported type from `gleam_http`.\n\n [`method_get`](#method_get), [`method_post`](#method_post), and\n [`method_options`](#method_options) are exposed in this namespace as\n constants. To check for other methods, you&#39;ll need to import\n [`gleam/http`](https://hexdocs.pm/gleam_http/gleam/http.html).\n\nSynonyms:\nmessua/handle.Method\nmessua/handle Method","url":"messua/handle.html#Method"},{"doc":"messua/handle","title":"Scheme","content":"pub type Scheme =\n  http.Scheme\n Reexported type from `gleam_http`.\n\nSynonyms:\nmessua/handle.Scheme\nmessua/handle Scheme","url":"messua/handle.html#Scheme"},{"doc":"messua/handle","title":"SockAddr","content":"pub type SockAddr {\n  SockAddr(addr: IpAddr, port: Int)\n}\n\nSockAddr(addr: IpAddr, port: Int)\n\n\nSynonyms:\nmessua/handle.SockAddr\nmessua/handle SockAddr","url":"messua/handle.html#SockAddr"},{"doc":"messua/handle","title":"method_get","content":"pub const method_get: Method\n Re-exported `gleam/http.Method` variant.\nSynonyms:\nmessua/handle.method_get\nmessua/handle method_get","url":"messua/handle.html#method_get"},{"doc":"messua/handle","title":"method_options","content":"pub const method_options: Method\n Re-exported `gleam/http.Method` variant.\nSynonyms:\nmessua/handle.method_options\nmessua/handle method_options","url":"messua/handle.html#method_options"},{"doc":"messua/handle","title":"method_post","content":"pub const method_post: Method\n Re-exported `gleam/http.Method` variant.\nSynonyms:\nmessua/handle.method_post\nmessua/handle method_post","url":"messua/handle.html#method_post"},{"doc":"messua/handle","title":"scheme_http","content":"pub const scheme_http: Scheme\n Re-exported `gleam/http.Scheme` variant.\nSynonyms:\nmessua/handle.scheme_http\nmessua/handle scheme_http","url":"messua/handle.html#scheme_http"},{"doc":"messua/handle","title":"scheme_https","content":"pub const scheme_https: Scheme\n Re-exported `gleam/http.Scheme` variant.\nSynonyms:\nmessua/handle.scheme_https\nmessua/handle scheme_https","url":"messua/handle.html#scheme_https"},{"doc":"messua/handle","title":"cookies","content":"pub fn cookies(req: MRequest(a)) -&gt; List(String)\n Return a list of all `Set-Cookie` header values.\nSynonyms:\nmessua/handle.cookies\nmessua/handle cookies","url":"messua/handle.html#cookies"},{"doc":"messua/handle","title":"fmt_sock_addr","content":"pub fn fmt_sock_addr(addr: SockAddr) -&gt; String\n Format a `SockAddr` like you&#39;d expect:\n\n \n SockAddr(addr: Ipv4(192, 168, 1, 1), port: 8080)\n |&gt; handle.fmt_sock_addr()\n // 192.168.1.1:8080\n  SockAddr(addr: Ipv6(10, 1534, 2020, 92, 7, 0, 0, 9), port: 443)\n  |&gt; handle.fmt_sock_addr()\n  |&gt; io.println()\n // [000A:05FE:07E4:005C:0007:0000:0000:0009]:443\n `\nSynonyms:\nmessua/handle.fmt_sock_addr\nmessua/handle fmt_sock_addr","url":"messua/handle.html#fmt_sock_addr"},{"doc":"messua/handle","title":"get_body","content":"pub fn get_body(\n  req: MRequest(a),\n  size_limit: Int,\n) -&gt; Option(BitArray)\n Return the body of the request if there is one, and it&#39;s less than\n `size_limit` bytes long.\nSynonyms:\nmessua/handle.get_body\nmessua/handle get_body","url":"messua/handle.html#get_body"},{"doc":"messua/handle","title":"get_client","content":"pub fn get_client(mreq: MRequest(a)) -&gt; Option(SockAddr)\n Return the client&#39;s `SockAddr` (if possible).\nSynonyms:\nmessua/handle.get_client\nmessua/handle get_client","url":"messua/handle.html#get_client"},{"doc":"messua/handle","title":"get_header","content":"pub fn get_header(\n  req: MRequest(a),\n  name: String,\n) -&gt; Option(String)\n\nSynonyms:\nmessua/handle.get_header\nmessua/handle get_header","url":"messua/handle.html#get_header"},{"doc":"messua/handle","title":"get_port","content":"pub fn get_port(req: MRequest(a)) -&gt; Option(Int)\n Gets the port to which the request was sent, not the port of the\n client&#39;s socket (use [`get_client()`](handle.html#get_client) for that).\nSynonyms:\nmessua/handle.get_port\nmessua/handle get_port","url":"messua/handle.html#get_port"},{"doc":"messua/handle","title":"get_query","content":"pub fn get_query(req: MRequest(a)) -&gt; Option(String)\n Return the request URL&#39;s query string (if there is one).\nSynonyms:\nmessua/handle.get_query\nmessua/handle get_query","url":"messua/handle.html#get_query"},{"doc":"messua/handle","title":"get_string_body","content":"pub fn get_string_body(\n  req: MRequest(a),\n  size_limit: Int,\n) -&gt; Option(String)\n Return the body of the request if there is one, it&#39;s less than\n `size_limit` bytes long, and is valid UTF-8.\nSynonyms:\nmessua/handle.get_string_body\nmessua/handle get_string_body","url":"messua/handle.html#get_string_body"},{"doc":"messua/handle","title":"header_map","content":"pub fn header_map(req: MRequest(a)) -&gt; Dict(String, String)\n Returns all request headers as a `Dict` that maps header names to\n header values.\n\n ### Note\n This will not produce the right value for multiple `Set-Cookie` headers;\n if you need to handle `Set-Cookie` headers, use `cookies()`.\nSynonyms:\nmessua/handle.header_map\nmessua/handle header_map","url":"messua/handle.html#header_map"},{"doc":"messua/handle","title":"headers","content":"pub fn headers(req: MRequest(a)) -&gt; List(#(String, String))\n Returns all request headers as a list of `#(name, value)` tuples.\nSynonyms:\nmessua/handle.headers\nmessua/handle headers","url":"messua/handle.html#headers"},{"doc":"messua/handle","title":"host","content":"pub fn host(req: MRequest(a)) -&gt; String\n\nSynonyms:\nmessua/handle.host\nmessua/handle host","url":"messua/handle.html#host"},{"doc":"messua/handle","title":"method","content":"pub fn method(req: MRequest(a)) -&gt; Method\n\nSynonyms:\nmessua/handle.method\nmessua/handle method","url":"messua/handle.html#method"},{"doc":"messua/handle","title":"parse_ip","content":"pub fn parse_ip(str: String) -&gt; Result(IpAddr, Nil)\n Attempt to parse an IPV4 or IPV6 address from a string.\n\n \n  handle.parse_ip(&quot;127.0.0.1&quot;)\n  |&gt; string.inspect()\n  |&gt; io.println()\n  // Ok(Ipv4(127, 0, 0, 1))\n \n  handle.parse_ip(&quot;32ff:40:0b0::3456&quot;)\n  |&gt; string.inspect()\n  |&gt; io.println()\n // Ok(Ipv6(13055, 64, 176, 0, 0, 0, 0, 13398))\n \nSynonyms:\nmessua/handle.parse_ip\nmessua/handle parse_ip","url":"messua/handle.html#parse_ip"},{"doc":"messua/handle","title":"path","content":"pub fn path(req: MRequest(a)) -&gt; String\n\nSynonyms:\nmessua/handle.path\nmessua/handle path","url":"messua/handle.html#path"},{"doc":"messua/handle","title":"path_segments","content":"pub fn path_segments(req: MRequest(a)) -&gt; List(String)\n Return the (non-empty) segments of a request path.\nSynonyms:\nmessua/handle.path_segments\nmessua/handle path_segments","url":"messua/handle.html#path_segments"},{"doc":"messua/handle","title":"require_body","content":"pub fn require_body(\n  req: MRequest(a),\n  size_limit: Int,\n  next: fn(BitArray) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Extract the raw bytes of the body of the request, or return a 400 with\n a message if the body is absent.\n\n ## Examples\n\n \n import messua/handle\n import messua/err\n import messua/ok\n\n const max_body_size: Int = 0x100000 // 1 MiB\n\n fn process_request(req: MRequest(state)) -&gt; MResponse {\n   use body_bytes &lt;- handle.require_body(req, max_body_size)\n\n   case process_body(body_bytes) {\n     Ok(result) -&gt; ok.ok()\n       |&gt; ok.with_binary_body(result)\n     Error(e) -&gt; err.new(500)\n       |&gt; err.with_message([&quot;unable to process request: &quot;, string.inspect(e)])\n   }\n }\n \nSynonyms:\nmessua/handle.require_body\nmessua/handle require_body","url":"messua/handle.html#require_body"},{"doc":"messua/handle","title":"require_client","content":"pub fn require_client(\n  mreq: MRequest(a),\n  next: fn(SockAddr) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Requires the client&#39;s `SockAddr` to be discernable, or returns a\n 500 error w/a log message attached.\nSynonyms:\nmessua/handle.require_client\nmessua/handle require_client","url":"messua/handle.html#require_client"},{"doc":"messua/handle","title":"require_header","content":"pub fn require_header(\n  req: MRequest(a),\n  name: String,\n  next: fn(String) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Get the given header value, or return a 400 response with a message\n if not present.\n\n ## Examples\n\n \n import gleam/result\n import messua/handle\n import messua/ok\n\n fn retrieve_user_data(req: MRequest) -&gt; MResponse {\n    use uname &lt;- handle.require_header(req, &quot;x-user-name&quot;)\n    \n    use user &lt;- result.try(\n      get_user_by_name(uname)\n      |&gt; result.replace_error(err.new(500))\n    )\n\n    let body = encode_user_details(user)\n\n    ok.ok()\n    |&gt; ok.with_json_body(body)\n    |&gt; Ok()\n }\n \nSynonyms:\nmessua/handle.require_header\nmessua/handle require_header","url":"messua/handle.html#require_header"},{"doc":"messua/handle","title":"require_json_body","content":"pub fn require_json_body(\n  req: MRequest(a),\n  size_limit: Int,\n  decoder: fn(Dynamic) -&gt; Result(b, List(DecodeError)),\n  next: fn(b) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Extract the body of the request and attempt to JSON from it with the\n given `gleam/dynamic` decoder. Returns a 400 if the body is missing or\n fails to decode with the given decoder.\n\n ## Examples\n\n \n import gleam/dynamic.{type Dynamic, type DecodeErrors}\n import messua/handle\n import messua/err\n import messua/ok\n\n type Book {\n   Book(\n     author: String,\n     title: String,\n     pub_year: Int,\n   )\n }\n\n fn book_decoder(chunk: Dynamic) -&gt; Result(Book, DecodeErrors) {\n   chunk\n   |&gt; dynamic.decode3(\n     Book,\n     dynamic.field(&quot;author&quot;, dynamic.string),\n     dynamic.field(&quot;title&quot;, dynamic.string),\n     dynamic.field(&quot;pub_year&quot;, dynamic.int)\n   )\n }\n\n fn read_books(req: MRequest) -&gt; MResponse {\n   use book_list &lt;- handle.require_json_body(\n     req,\n     0x1000, // 4 KiB of `Book`s\n     dynamic.list(book_decoder)\n   )\n\n   case fetch_and_consume_these_books(book_list) {\n     Ok(_) -&gt; ok.ok() |&gt; Ok()    // Okay, okay, okay!!!\n     Error(_) -&gt; err.new(500)\n       |&gt; err.with_message([&quot;Couldn&#39;t read them all! :^(&quot;])\n       |&gt; Error()\n   }\n }\n \nSynonyms:\nmessua/handle.require_json_body\nmessua/handle require_json_body","url":"messua/handle.html#require_json_body"},{"doc":"messua/handle","title":"require_query","content":"pub fn require_query(\n  req: MRequest(a),\n  next: fn(String) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Return the request URL&#39;s query string, or answer with a 400 response.\nSynonyms:\nmessua/handle.require_query\nmessua/handle require_query","url":"messua/handle.html#require_query"},{"doc":"messua/handle","title":"require_string_body","content":"pub fn require_string_body(\n  req: MRequest(a),\n  size_limit: Int,\n  next: fn(String) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Like `require_body`, but returns a `String`; will also fail with\n a 400 if the body isn&#39;t valid UTF-8.\nSynonyms:\nmessua/handle.require_string_body\nmessua/handle require_string_body","url":"messua/handle.html#require_string_body"},{"doc":"messua/handle","title":"require_valid_header","content":"pub fn require_valid_header(\n  req: MRequest(a),\n  name: String,\n  validator: fn(String) -&gt; Result(b, c),\n  next: fn(b) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Requires the given header to both exist and have a value acceptable\n to a supplied validator function. A 400 result with an explanatory\n message is returned on failure.\n\n ## Examples\n\n \n import gleam/int\n import gleam/result\n import messua/handle\n import messua/ok\n\n fn retrieve_user_data(req: MRequest) -&gt; MResponse {\n    use uid &lt;- handle.require_valid_header(req, &quot;x-user-name&quot;, int.parse)\n    \n    use user &lt;- result.try(\n      get_user_by_id(uid)\n      |&gt; result.replace_error(err.new(500))\n    )\n\n    let body = encode_user_details(user)\n\n    ok.ok()\n    |&gt; ok.with_json_body(body)\n    |&gt; Ok()\n }\n \nSynonyms:\nmessua/handle.require_valid_header\nmessua/handle require_valid_header","url":"messua/handle.html#require_valid_header"},{"doc":"messua/handle","title":"scheme","content":"pub fn scheme(req: MRequest(a)) -&gt; Scheme\n\nSynonyms:\nmessua/handle.scheme\nmessua/handle scheme","url":"messua/handle.html#scheme"},{"doc":"messua/handle","title":"messua/handle","content":" Helper functions for use in handlers.\n\n * The functions in this module whose names start with `require_`\n   are intended for `use` expressions.\n * The functions in this module whose names start with `get_` return\n   `Option`s.\n * Functions whose names are just nouns will always return valid\n   (although possibly empty or meaningless) values.","url":"messua/handle.html"},{"doc":"messua/mware","title":"LogTarget","content":"pub type LogTarget {\n  Stdout\n  Stderr\n  File(path: String)\n}\n For specifying where the `log_layer()` should output its logs to.`\nStdout\n The standard output.\n\nStderr\n The standard error stream.\n\nFile(path: String)\n Append to the file at the given path.\n\nSynonyms:\nmessua/mware.LogTarget\nmessua/mware LogTarget","url":"messua/mware.html#LogTarget"},{"doc":"messua/mware","title":"make_blacklist_layer","content":"pub fn make_blacklist_layer(\n  addrs: List(IpAddr),\n) -&gt; fn(\n  MRequest(a),\n  fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Return a layer that rejects requests from all listed addresses with\n an unceremonious 404.\n\n You can use [`handle.parse_ip()`](handle.html#parse_ip) to help read\n a list of strings (like from a configuration file).\n\n Plans for a future version include a way to specify a _range_ of\n IP addresses.\n\n ## Examples\n\n \n import gleam/list\n import gleam/result\n import messua/handle\n import messua/mware\n\n let assert Ok(blacklist) = [\n   &quot;173.194.219.100&quot;,\n   &quot;173.194.219.101&quot;,\n   &quot;173.194.219.102&quot;,\n   &quot;173.194.219.113&quot;,\n   &quot;173.194.219.138&quot;,\n   &quot;173.194.219.139&quot;,\n ]\n |&gt; list.map(handle.parse_ip)\n |&gt; result.all()\n\n let blacklist_layer = mware.make_blacklist_layer(blacklist)\n \nSynonyms:\nmessua/mware.make_blacklist_layer\nmessua/mware make_blacklist_layer","url":"messua/mware.html#make_blacklist_layer"},{"doc":"messua/mware","title":"make_compression_layer","content":"pub fn make_compression_layer(\n  min_size: Int,\n) -&gt; fn(\n  MRequest(a),\n  fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Return a `Layer` that compresses outgoing responses if they are at least\n `min_size` bytes and the incoming request contained an\n `Accept-Encoding: deflate` header.\n\n Like [`ok.compress_body()`](ok.html#compress_body), this does not do\n anything to responses generated with\n [`ok.serve_file()`](ok.html#serve_file) or\n [`ok.serve_dir()`](ok.html.#serve_dir).\nSynonyms:\nmessua/mware.make_compression_layer\nmessua/mware make_compression_layer","url":"messua/mware.html#make_compression_layer"},{"doc":"messua/mware","title":"make_log_layer","content":"pub fn make_log_layer(\n  target: LogTarget,\n) -&gt; fn(\n  MRequest(a),\n  fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Return a `Layer` function that logs to the specified target.\n\n Currently, logging to a file opens the file for append each time.\n This is clearly not the most efficient method, but I don&#39;t know enough\n Erlang yet to do better.\n\n TODO: Learn some Erlang and do better.\n\n ## Safety\n If you specify a path that cannot be written to, the server will\n crash upon startup.\nSynonyms:\nmessua/mware.make_log_layer\nmessua/mware make_log_layer","url":"messua/mware.html#make_log_layer"},{"doc":"messua/mware","title":"make_rate_limit_layer","content":"pub fn make_rate_limit_layer(\n  requests: Int,\n  over_seconds: Int,\n  cull_expired_counters_every: Int,\n) -&gt; fn(\n  MRequest(a),\n  fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Return a layer that limits requests from any given IP address to\n `requests` requests every `over_seconds` seconds. Excess requests will\n be rejected with a a 429.\n\n 10 requests every two seconds and 20 requests every four seconds both\n limit to the same average rate (one request every 200 ms), but the\n latter setting allows for larger &quot;bursts&quot; of requests (with, of course,\n a correspondingly longer cooldown).\n\n The `cull_expired_counters_every` parameter controls how often the\n layer will crawl through and throw away records from clients that\n haven&#39;t send a request in `over_seconds` seconds or more.\n\n ## Examples\n\n \n import messua\n import messua/mware\n import messua/rr\n\n fn inner_handler(req: rr.MRequest) -&gt; rr.MResponse {\n   // Your routing and business logic goes here.\n }\n\n // 2 requests/second, with bursts up to 12 requests.\n let limit_layer = mware.make_rate_limit_layer(12, 6, 1024)\n\n // Wrap our `inner_handler` in our `limit_layer` to make our\n // composite handler.\n let handler = limit_layer(_, inner_handler)\n\n messua.default()\n   |&gt; messua.start(handler)\n \nSynonyms:\nmessua/mware.make_rate_limit_layer\nmessua/mware make_rate_limit_layer","url":"messua/mware.html#make_rate_limit_layer"},{"doc":"messua/mware","title":"make_static_header_layer","content":"pub fn make_static_header_layer(\n  headers: List(#(String, String)),\n) -&gt; fn(\n  MRequest(a),\n  fn(MRequest(a)) -&gt; Result(Response(ResponseData), Err),\n) -&gt; Result(Response(ResponseData), Err)\n Adds the given list of `#(name, value)` headers to each response.\nSynonyms:\nmessua/mware.make_static_header_layer\nmessua/mware make_static_header_layer","url":"messua/mware.html#make_static_header_layer"},{"doc":"messua/mware","title":"messua/mware","content":" Some basic middlware layers.\n\n See the documentation for \n [`Layer`](rr.html#Layer) and even more so the\n [Stack example](https://codeberg.org/d2718/messua/src/branch/main/examples/stack)\n for demonstrations of how to put these together.","url":"messua/mware.html"}]);